# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""Constants"""
TOKEN = 'bf186b0a-ff34-44b2-9f62-496aa0d5e2c7'
HEADERS = {
    'text': {'Content-Type': 'text/plain'},
    'json': {'Content-Type': 'application/json'}
}
